#include "graph2.hh"
#include <sys/time.h>

static double getNow()
{
  struct timeval tv;
  gettimeofday(&tv,0);
  return tv.tv_sec+tv.tv_usec/1000000.0;
}  


void MyGraphView::on_realize()
{
  Gtk::DrawingArea::on_realize();
  d_win=get_window();

  Gdk::Color color;
  d_bg=Gdk::GC::create(d_win);
  d_bg->set_rgb_fg_color(color);

  color.set_green(65535);
  d_first=Gdk::GC::create(d_win);
  d_first->set_rgb_fg_color(color);

  color.set_red(65535);
  d_grid=Gdk::GC::create(d_win);
  d_grid->set_rgb_fg_color(color);

  d_realized=true;
  Glib::signal_timeout().connect( SigC::slot(*this, &MyGraphView::on_timeout), 76);
  signal_configure_event().connect( SigC::slot(*this, &MyGraphView::on_configure_event));
}

bool MyGraphView::on_timeout()
{
  if(!d_realized)
    return false;

  double now=getNow();
  d_last=now-20.0;
  int x=0, y, lastx, lasty;


  // clear the backing store
  d_scribble->draw_rectangle(d_bg,true,0,0,get_width(),get_height());


  // draw grid on backing store
  for(int i=0;i<3;++i) {
    d_scribble->draw_line(d_grid,0,(i+1)*get_height()/4,get_width(), (i+1)*get_height()/4);
  }

  // draw the graph
  for(vector<pair<double,int> >::reverse_iterator i=d_points.rbegin();i!=d_points.rend() && i->first >= d_last;++i) {

    y=(int)(get_height()*(1-(1.0*i->second/d_ylimit)));
    x=(int)(get_width()*(1-(now-i->first)/20.0));
  
    if(i!=d_points.rbegin())
      d_scribble->draw_line(d_first,lastx,lasty,x,y);
  
    lastx=x;
    lasty=y;
  }

  queue_draw();

  return true;
}

void MyGraphView::addPoint(int val) 
{
  d_points.push_back(make_pair(getNow(),val));
}

bool MyGraphView::on_configure_event(GdkEventConfigure*)
{
  d_scribble = Gdk::Pixmap::create(
			   get_window(),
			   get_allocation().width,
			   get_allocation().height,
      -1);

  return true;
}

bool MyGraphView::on_expose_event(GdkEventExpose *g)
{
  d_win->draw_drawable(d_bg,d_scribble,0,0,0,0);  // write the backing store
  return true;
}

class GraphApp : public Gtk::Window
{
public:
  virtual ~GraphApp(){}
  GraphApp()
  {
    Glib::signal_timeout().connect( SigC::slot(*this, &GraphApp::on_timeout), 75);    
    add(d_mgv);
    show_all_children();
  }
  bool on_timeout()
  {
    d_mgv.addPoint(random()%50);
    return true;
  }
private:
  MyGraphView d_mgv;
};


int main(int argc, char **argv)
{
  Gtk::Main kit(&argc, &argv);

  GraphApp ma;
  kit.run(ma);
}
